@@ -20,6 +20,12 @@
 extern "C" {
 #endif
 
+#ifndef flint_abort
+#if __FLINT_RELEASE <= 20502
+#define flint_abort abort
+#endif
+#endif
+
 /* currently defined in the arb module, but global to the library */
 double arb_test_multiplier(void);
 
@@ -166,6 +172,20 @@ _fmpz_size(const fmpz_t f)
         return mpz_size(COEFF_TO_PTR(d));
 }
 
+static __inline__ void
+fmpz_ui_mul_ui(fmpz_t r, ulong a, ulong b)
+{
+    if (a < (UWORD(1) << (FLINT_BITS / 2)) && b < (UWORD(1) << (FLINT_BITS / 2)))
+    {
+        fmpz_set_ui(r, a * b);
+    }
+    else
+    {
+        fmpz_set_ui(r, a);
+        fmpz_mul_ui(r, r, b);
+    }
+}
+
 static __inline__ void
 fmpz_ui_pow_ui(fmpz_t x, ulong b, ulong e)
 {
